/* ``The contents of this file are subject to the Erlang Public License,
 * Version 1.0, (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.erlang.org/EPL1_0.txt
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Erlang-4.7.3, December, 1998.
 * 
 * The Initial Developer of the Original Code is Ericsson Telecom
 * AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
 * Telecom AB. All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.''
 */
#ifndef _VAR_H
#define _VAR_H

/* RCS: $Id: var.h,v 1.7 1996/04/09 13:23:44 tobbe Exp tobbe $ */

#include <sys/types.h>
#include <sys/stat.h>
#include <utime.h>
#include <limits.h>
#include "ig.h"

/* Test example:
 * 
 * 1> Z=var:start(). 
 * <0.87.0>
 * 2> var:x_int(Z). 
 * {ok,99}
 * 3> var:x_int(Z,123). 
 * ok
 * 4> var:x_int(Z). 
 * {ok,123}
 * 5> var:s_int(Z). 
 * {ok,0}
 * 6> var:s_int(Z,999). 
 * ok
 * 7> var:s_int(Z). 
 * {ok,999}
 * 8> var:s_int(Z,2147483647).
 * ok
 * 9> var:s_int(Z).           
 * {ok,2147483647}
 * 10> var:s_int(Z,-2147483647).
 * ok
 * 11> var:s_int(Z).            
 * {ok,-2147483647}
 * 12> catch var:s_int(Z,2147483648).
 * {'EXIT',{function_clause,{var,s_int,[<0.11.0>,2147483648]}}}
 * 13> catch var:s_int(Z,-2147483648).
 * {'EXIT',{function_clause,{var,s_int,[<0.11.0>,-2147483648]}}}
 * 14> {_,P}=ig:adr_of(Z,x_int). 
 * {ok,41304}
 * 15> var:get_var(Z,P). 
 * {ok,123}
 * 16> ig:sizeof(Z,x_int). 
 * {ok,4}
 * 17> {ok,Biggest} = var:get_biggest(Z).
 * {ok,4294967295}
 * 18> var:s_uint(Z,Biggest).
 * ok
 * 19> var:s_uint(Z). 
 * {ok,4294967295} 
 * 20> catch var:s_uint(Z,4294967296).
 * {'EXIT',{function_clause,{var,s_uint,[<0.11.0>,4294967296]}}}
 * 21> catch var:s_uint(Z,-1).         
 * {'EXIT',{function_clause,{var,s_uint,[<0.11.0>,-1]}}}
 * 22> {_,My_stat}=ig:adr_of(Z,my_stat).
 * {ok,42336}
 * 23> var:stat(Z,"/.cshrc",My_stat).
 * {ok,0}
 * 24> var:my_stat(Z).
 * #ifdef SUNOS4
 * {ok,{'struct stat',-32256,64225,33188,20,0,0,6580,495,
 *      809170184,0,674984854,0,808271809,0,8192,2,#Bin}}
 * #elif SOLARIS
 * {ok,{'struct stat',40894465,#Bin,204259,33188,36,0,10,4294949397,
 *      #Bin,528,0,833293168,827837206,833289368,8192,2,#Bin,#Bin}}
 * #endif
 * 25> var:my_utime(Z).
 * {ok,{'struct utimbuf',0,0}}
 * 26> var:my_utime(Z,{'struct utimbuf',809170184,674984854}).
 * ok
 * 27> var:my_utime(Z) 
 * {ok,{'struct utimbuf',809170184,674984854}}
 */

/*IG_var*/ extern int x_int;

/*IG_var*/ extern int s_int;

/*IG_var*/ extern unsigned s_uint;

IG_fun int get_var(int *);

IG_fun unsigned long get_biggest(void);

#ifdef HIDE

#ifdef SUNOS4
/* We have to uncomment this since IG don't know about #ifdef's

typedef long unsigned int size_t;
typedef long int off_t;
typedef unsigned short mode_t;
typedef short dev_t;
typedef unsigned short uid_t;
typedef unsigned short gid_t;
typedef unsigned long ino_t;
typedef unsigned short mode_t;  
typedef long time_t;           

struct stat { 
        dev_t   st_dev;
        ino_t   st_ino;
        mode_t  st_mode;
        short   st_nlink;
        uid_t   st_uid;
        gid_t   st_gid;
        dev_t   st_rdev;
        off_t   st_size;
        time_t  st_atime;  
        int     st_spare1;
        time_t  st_mtime;  
        int     st_spare2;
        time_t  st_ctime;  
        int     st_spare3;
        long    st_blksize;
        long    st_blocks;
        long    st_spare4[2];
};
*/
#endif

#ifdef SOLARIS
typedef long off_t;
typedef unsigned long ulong_t;
typedef ulong_t mode_t;
typedef long uid_t;
typedef uid_t gid_t;
typedef ulong_t nlink_t;
typedef ulong_t ino_t;
typedef ulong_t dev_t;
typedef long time_t;

#define _ST_FSTYPSZ 16          /* array size for file system type name */

struct  stat { /* Copy of the standard: struct stat. */
        dev_t   st_dev;
        long    st_pad1[3];     /* reserve for dev expansion, */
  /* sysid definition */
        ino_t   st_ino;
        mode_t  st_mode;
        nlink_t st_nlink;
        uid_t   st_uid;
        gid_t   st_gid;
        dev_t   st_rdev;
        long    st_pad2[2];
        off_t   st_size;
        long    st_pad3;        /* reserve pad for future off_t expansion */
        time_t  st_atime;
        time_t  st_mtime;
        time_t  st_ctime;
        long    st_blksize;
        long    st_blocks;
        char    st_fstype[16];  /* _ST_FSTYPSZ */
        long    st_pad4[8];     /* expansion area */
};
#endif

struct utimbuf {          /* Copy of the standard: struct utimebuf */
  time_t  actime;         /* set the access time */
  time_t  modtime;        /* set the modification time */
};

#endif

IG_var static struct stat my_stat;
IG_var static struct utimbuf my_utime;

IG_fun int stat(const IG_string path, struct stat *buf);

#endif
