%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%% Copyright (C) 1995, Ellemtel Telecommunications Systems Laboratories
%%-----------------------------------------------------------------------
%% Created:  21 Feb 1995 by tobbe@erix.ericsson.se
%% Function: Contains IG stuff.
%%-----------------------------------------------------------------------
%% RCS: $Id: ig.hrl,v 1.10 1996/04/10 06:24:55 tobbe Exp $

%% A member in a struct definition where:
%%   member   ::== (Descriptor x Name)
%%   Name     ::= <atom> | {<atom>,Constant}
%%   Constant ::= <integer>
-record(member,{desc,name}).

%% A type descriptor where:
%%  cat           ::= std | ptr | type | 'IG_string' | 
%%                    binaryPtr | assumed_ptr
%%  storage_class ::= extern |static | register | auto
%%  ansi_str      ::= <the (ANSI) type as a string>
%%  tname         ::= <the name of the type when std==type>
-record(descriptor,{cat,storage_class,ansi_string,tname}).

%% Symbol table entry where:
%%  name - is the symbol name as a string
%%  no   - is a unique number
%%  tag  - identifies the symbol
-record(symbol,{name,no,tag}).

%% Some C keywords
-define( STD_C_TYPES , [int,char,short,long,signed,unsigned,float,double,void] ).
-define( STORAGE_CLASS , [extern,static,register,auto] ).
-define( TYPE_QUALIFIERS , [const,volatile] ).

%% Tags for identifying the type of callback.
%% They are also defined in ig.h
-define(IG_FUN_TAG,    0). 
-define(IG_CALL_TAG,   1). 
-define(IG_CAST_TAG,   2).
-define(IG_EXTERN_TAG, 3).
-define(IG_VOID_TAG,   4).
-define(IG_VAR_TAG,    5).
-define(IG_SIZE_TAG,   6).

%% The variable names of the communication fd's
-define(IG_FUN_IN,    "ig_fun_in").
-define(IG_FUN_OUT,   "ig_fun_out").
-define(IG_CBACK_IN,  "ig_cback_in").
-define(IG_CBACK_OUT, "ig_cback_out").

%% Variable containing error codes
-define(IG_ERRNO, "ig_errno").

%% The size of the IG local buffer
-define(BUFSIZE, 8192).

%% Short forms
-define(lc, lists:concat).
-define(ETERM, "ETERM").
-define(ETERM_PTR, "ETERM *").
-define(uint, "unsigned int").
-define(nl, "\n").

%% TTY routine
-ifdef(silent).
-define(TELL(X,Y), true).
-define(TICK, true).
-else.
-define(TELL(X,Y), io:format(X,Y)).
-define(TICK, io:format(".")).
-endif.

%% Debug routine
-ifdef(debug).
-define(DBG(X,Y), io:format(X,Y)).
-define(DBG1(X), X).
-else.
-define(DBG(X,Y), true).
-define(DBG1(X), true).
-endif.

%% Nice functions inspired from Oz
-define(andthen(C,T), if C -> T; true -> false end).
-define(orelse(C,T),  if C -> true; true -> T end).

%% A null pointer
-define(NULL, {0,0,0,0}).

%% Used in the 'sizeof handling' of variables
-define(SIZE_PREFIX, "__").
