%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%% Version in the SNMP Toolkit (as a string())
-define(version, "2.2").

%%----------------------------------------------------------------------
%% Note: All internal representations may be changed without notice.
%%----------------------------------------------------------------------

%%----------------------------------------------------------------------
%% Variablebinding
%% oid: a list of integers (see misc:is_oid)
%% variabletype corresponds to type in the asn1_type.
%% variabletype=='NULL' =>
%%     value== 'NULL' | noSuchObject | noSuchInstance | endOfMibView
%% else: variabletype == <one of the four types occuring
%%                        in SimpleSyntax (see rfc 1142)>
%% value: a value.
%% org_index: an integer. Its position in the original varbindlist (the one
%%            from the get- or set-request).
%%----------------------------------------------------------------------
-record(varbind, {oid, variabletype, value, org_index}).

%%-----------------------------------------------------------------
%% Internal Variablebinding
%% status = noError | ErrorStatus
%% mibentry = A mibentry if status == noError
%% varbind = a varbind-record
%%-----------------------------------------------------------------
-record(ivarbind, {status = noError, mibentry, varbind}).

%%----------------------------------------------------------------------
%% ASN1_type. Everything that is needed to represent a typed variable.
%% BERtype: Type used during Basic Encoding/Decoding Rules
%% aliasname: The name of the derived type as defined in the MIB
%% assocList can be list of:
%% {enums, [{up, 1}, {down, 2}, {right, 3}, {left, 4}]}
%%----------------------------------------------------------------------
-record(asn1_type, {bertype, lo, hi, assocList = [], imported = false,
		    aliasname, implied = false}).


%%----------------------------------------------------------------------
%% MibEntry
%% aliasname is the name for the oid.
%% asn1_type is a record of asn1_type.
%% entrytype: variable | table | tableColumn | internal
%% access: notAccessible | readOnly | readWrite | readCreate     (see rfc 1142)
%% assocList: list of
%% {table_info, #table_info}      when entrytype == table
%% {varable_info, #variable_info} when entrytype == variable
%% {table_name, TableName}        when entrytype == table_column
%% {table_entry_with_sequence, NameOfSequence} when entrytype == table_entry
%%----------------------------------------------------------------------
-record(me, {oid, entrytype, aliasname, asn1_type,
	     access, mfa, imported = false, assocList = []}).

%% oidobjects is a list of {oid, asn1_type} to be sent in the trap.
-record(trap, {trapname, enterpriseoid, specificcode, oidobjects}).

%% oidobjects is a list of {oid, asn1_type} to be sent in the trap.
-record(notification, {trapname, oid, oidobjects}).

%%----------------------------------------------------------------------
%% This is how a mib is represented on disk (as a binary)
%% types is: [asn1_type()]
%% variable_infos is a list of {Name,  variable_info-record}
%% table_infos is a list of {Name,  table_info-record}
%%----------------------------------------------------------------------
-record(mib, {misc = [], mib_format_version = "2.0", name = "",
              mes = [], asn1_types = [], traps = [], variable_infos = [],
              table_infos = []}).

%%----------------------------------------------------------------------
%%  data is a PDU.
%%----------------------------------------------------------------------
-record(message, {version, community, data}).

%%----------------------------------------------------------------------
%% type: 'get-request' | 'get-next-request' | 'get-bulk-request' |
%% 'get-response' | 'set-request' | 'inform-request' | 'snmpv2-trap' | report
%% (see rfc 1905)
%% request_id, error_status and error_index are integers.
%% varbinds: a list of varbinds.
%%----------------------------------------------------------------------
%%               if bulk        non-repeaters max-repetitions  resp
-record(pdu, {type, request_id, error_status, error_index, varbinds}).

-record(trappdu, {enterprise, agent_addr, generic_trap, specific_trap,
		  time_stamp, varbinds}).

%%-----------------------------------------------------------------
%% This record should be used when a Mnesia table for variables
%% is created.
%%-----------------------------------------------------------------
-record(snmp_variables, {name, value}).

%%-----------------------------------------------------------------
%% The OTP Security Model (ericsson * 256 + otp)
%%-----------------------------------------------------------------
-define(OTP_SEC, 49427).
